@@ -388,7 +388,7 @@ class TestScripts(unittest.TestCase):
             src = f.read()
         ast.parse(src)
 
-    def test_check_presence(self):
+    def test_coverage(self):
         # make sure all example scripts have a test method defined
         meths = dir(self)
         for name in os.listdir(SCRIPTS_DIR):
@@ -418,8 +418,9 @@ class TestScripts(unittest.TestCase):
     def test_procinfo(self):
         self.assert_stdout('procinfo.py', args=str(os.getpid()))
 
-    # can't find users on APPVEYOR
-    @unittest.skipIf(APPVEYOR, "unreliable on APPVEYOR")
+    # can't find users on APPVEYOR or TRAVIS
+    @unittest.skipIf(APPVEYOR or TRAVIS and not psutil.users(),
+                     "unreliable on APPVEYOR or TRAVIS")
     def test_who(self):
         self.assert_stdout('who.py')
 
@@ -465,6 +466,22 @@ class TestScripts(unittest.TestCase):
     def test_winservices(self):
         self.assert_stdout('winservices.py')
 
+    def test_cpu_distribution(self):
+        self.assert_syntax('cpu_distribution.py')
+
+    def test_temperatures(self):
+        if hasattr(psutil, "sensors_temperatures"):
+            self.assert_stdout('temperatures.py')
+        else:
+            self.assert_syntax('temperatures.py')
+
+    def test_battery(self):
+        if hasattr(psutil, "sensors_battery") and \
+                psutil.sensors_battery() is not None:
+            self.assert_stdout('battery.py')
+        else:
+            self.assert_syntax('battery.py')
+
 
 # ===================================================================
 # --- Unit tests for test utilities.
